﻿//Project $safeprojectname$
//Created by Björn Dagerman 2015/05 (dagerman@kth.se)
// Fixed typo (changed FSharp -> FSharpx), Björn L 2017-02-27
//
//Main
namespace Lab4
open FSharpx.Control.Observable //Fsharpx

module Main =
    //This main function loops using async and Async.Await. See lecture F13 for alternatives.
    let rec loop observable list = async{
        //At the start we do the computations that we can do with the inputs we have, just as in a regular application
        printfn "%A" list
    
        //Next, since we don't have all inputs (yet) we need to wait for them to become available (Async.Await)
        //let! (bang) enables execution to continue on other computations and threads.
        let! somethingObservable = Async.AwaitObservable(observable) 

        //Now that we have recieved a new input we can perform the rest of our computations
        match somethingObservable with
            | 0     -> return! loop observable (0::list)
            | 1     -> return! loop observable (1::list)
            | _     -> failwith "Not implemented yet!"
        //The last thing we do is a recursive call to ourselves, thus looping
    }

    //The GuiInterface is tightly coupled with the main loop which is its only intented user, thus the nested module
    module GuiInterface = 
        //Here we define what we will be observing (clicks)
        let observables = 
            Observable.merge 
            <| Observable.map (fun _-> 0) Gui.btnAdd0.Click //The map transforms the observation (click) by the given function. In our case this means
            <| Observable.map (fun _-> 1) Gui.btnAdd1.Click //that clicking the button AddX will return X. Note the type of observables : IObservable<int>

    //Starts the main loop and opens the Gui 
    Async.StartImmediate(loop GuiInterface.observables []); System.Windows.Forms.Application.Run(Gui.form)


